@@ -180,6 +180,38 @@ writev_bucket(write_bucket *data)
     return 1;
 }
 
+static inline void
+auto_set_content_length(client_t *client, write_bucket *bucket, char *data, size_t datalen )
+{
+    PyObject *header, *length;
+    char *value;
+    Py_ssize_t valuelen;
+
+    if(client->headers && !client->content_length_set){
+        if (PySequence_Check(client->response)) {
+            if (PySequence_Size(client->response) == 1) {
+                client->content_length_set = 1;
+#ifdef DEBUG
+                printf("set content_length %d \n", datalen);
+#endif
+                length = PyString_FromFormat("%zu", datalen);
+
+                header = Py_BuildValue("(sO)", "Content-Length", length);
+                Py_DECREF(length);
+
+                PyList_Append(client->headers, header);
+                Py_DECREF(header); 
+                PyString_AsStringAndSize(length, &value, &valuelen);
+                add_header(bucket, "Content-Length", 14, value, valuelen);
+            }
+
+            if (PyErr_Occurred()){
+                PyErr_Clear();
+            }
+        }
+    }
+}
+
 static inline int
 write_headers(client_t *client, char *data, size_t datalen)
 {
@@ -293,6 +325,14 @@ write_headers(client_t *client, char *data, size_t datalen)
         }
         
     }
+    //header done 
+    
+    // check content_length_set
+    // auto set content_length
+    if(data){
+        auto_set_content_length(client, bucket, data, datalen); 
+    }
+
     set2bucket(bucket, CRLF, 2);
     if(data){
         set2bucket(bucket, data, datalen);
@@ -341,27 +381,29 @@ write_sendfile(int out_fd, int in_fd, size_t count)
 inline void 
 close_response(client_t *client)
 {
-    
-    //send all response
-    //closing reponse object
-    if (client->response && PyObject_HasAttrString(client->response, "close")) {
-        PyObject *close = NULL;
-        PyObject *args = NULL;
-        PyObject *data = NULL;
-        
-        close = PyObject_GetAttrString(client->response, "close");
+    if(!client->response_closed){ 
+        //send all response
+        //closing reponse object
+        if (client->response && PyObject_HasAttrString(client->response, "close")) {
+            PyObject *close = NULL;
+            PyObject *args = NULL;
+            PyObject *data = NULL;
+            
+            close = PyObject_GetAttrString(client->response, "close");
 
-        args = Py_BuildValue("()");
-        data = PyEval_CallObject(close, args);
+            args = Py_BuildValue("()");
+            data = PyEval_CallObject(close, args);
 
-        Py_DECREF(args);
-        Py_XDECREF(data);
-        Py_DECREF(close);
-        if (PyErr_Occurred()){
-            PyErr_Clear();
+            Py_DECREF(args);
+            Py_XDECREF(data);
+            Py_DECREF(close);
+            if (PyErr_Occurred()){
+                PyErr_Clear();
+            }
         }
+
+        client->response_closed = 1;
     }
-    client->response_closed = 1; 
 
 }
 
@@ -529,14 +571,10 @@ start_response_write(client_t *client)
     char *buf;
     Py_ssize_t buflen;
     
-    if(PyIter_Check(client->response)){
-        iterator = client->response;
-    }else{
-        iterator = PyObject_GetIter(client->response);
-        if (PyErr_Occurred()){ 
-            write_error_log(__FILE__, __LINE__);
-            return -1;
-        }
+    iterator = PyObject_GetIter(client->response);
+    if (PyErr_Occurred()){ 
+        write_error_log(__FILE__, __LINE__);
+        return -1;
     }
     client->response_iter = iterator;
 
