@@ -42,6 +42,8 @@ static char *error_log_path = NULL; //error log path
 static int err_log_fd = -1; //error log
 
 static int is_keep_alive = 0; //keep alive support
+//static int keep_alive_timeout = 5;
+
 int max_content_length = 1024 * 1024 * 16; //max_content_length
 
 static char *unix_sock_name = NULL;
@@ -134,6 +136,7 @@ close_conn(client_t *cli, picoev_loop* loop)
     }else{
         disable_cork(cli);
         new_client = new_client_t(cli->fd, cli->remote_addr, cli->remote_port);
+        new_client->keep_alive = 1;
         init_parser(new_client, server_name, server_port);
         picoev_add(main_loop, new_client->fd, PICOEV_READ, READ_LONG_TIMEOUT_SECS, r_callback, (void *)new_client);
     }
@@ -456,15 +459,13 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
         }
         r = read(cli->fd, buf, sizeof(buf));
         switch (r) {
-            /*
-            read 0 occured overload
             case 0: 
                 cli->keep_alive = 0;
+                //503??
                 cli->status_code = 503;
                 send_error_page(cli);
                 close_conn(cli, loop);
                 return;
-            */
             case -1: /* error */
                 if (errno == EAGAIN || errno == EWOULDBLOCK) { /* try again later */
                     break;
@@ -476,7 +477,6 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
                     if(errno != ECONNRESET){
                         send_error_page(cli);
                     }else{
-
                         cli->header_done = 1;
                         cli->response_closed = 1;
                     }
@@ -501,13 +501,7 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
                     return;
                 }
 
-                if(cli->upgrade){
-                    //WebSocket Key
-                    char *key = buf + nread + 1;
-                    PyObject *body = PyString_FromStringAndSize(key, r - nread);
-                    cli->body = body;
-
-                }else if(!cli->upgrade && nread != r){
+                if(!cli->upgrade && nread != r){
                     // parse error
 #ifdef DEBUG
                     printf("fd %d parse error %d \n", cli->fd, cli->bad_request_code);
@@ -523,6 +517,13 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 #endif
                
                 if(parser_finish(cli) > 0){
+                    if(cli->upgrade){
+                        //WebSocket Key
+                        char *key = buf + nread + 1;
+                        PyObject *body = PyString_FromStringAndSize(key, r - nread);
+                        cli->body = body;
+
+                    }
                     finish = 1;
                 }
                 break;
@@ -764,10 +765,21 @@ meinheld_access_log(PyObject *self, PyObject *args)
 {   
     if (!PyArg_ParseTuple(args, "s:access_log", &log_path))
         return NULL;
+    
 
     if(log_fd > 0){
         close(log_fd);
     }
+    
+    if(!strcasecmp(log_path, "stdout")){
+        log_fd = 1;
+        Py_RETURN_NONE;
+    }
+    if(!strcasecmp(log_path, "stderr")){
+        log_fd = 2;
+        Py_RETURN_NONE;
+    }
+
     log_fd = open_log_file(log_path);
     if(log_fd < 0){
         PyErr_Format(PyExc_TypeError, "not open file. %s", log_path);
@@ -848,6 +860,9 @@ meinheld_run_loop(PyObject *self, PyObject *args)
             Py_XDECREF(watchdog_result);
             i = 0;
         }
+#ifdef DEBUG
+        printf("loop \n");
+#endif
     }
 
     Py_DECREF(wsgi_app);
@@ -869,12 +884,18 @@ meinheld_run_loop(PyObject *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
-/*
+
 PyObject *
 meinheld_set_keepalive(PyObject *self, PyObject *args)
 {
-    if (!PyArg_ParseTuple(args, "i", &is_keep_alive))
+    int on;
+    if (!PyArg_ParseTuple(args, "i", &on))
         return NULL;
+    if(on < 0){
+        PyErr_SetString(PyExc_ValueError, "must be 0 or 1");
+        return NULL;
+    }
+    is_keep_alive = on;
     Py_RETURN_NONE;
 }
 
@@ -883,7 +904,7 @@ meinheld_get_keepalive(PyObject *self, PyObject *args)
 {
     return Py_BuildValue("i", is_keep_alive);
 }
-*/
+
 
 PyObject *
 meinheld_set_max_content_length(PyObject *self, PyObject *args)
@@ -1062,10 +1083,13 @@ static PyMethodDef WsMethods[] = {
     {"listen", meinheld_listen, METH_VARARGS, "set host and port num"},
     {"access_log", meinheld_access_log, METH_VARARGS, "set access log file path."},
     {"error_log", meinheld_error_log, METH_VARARGS, "set error log file path."},
-    //{"set_keepalive", meinheld_set_keepalive, METH_VARARGS, "support keep-alive. set 1"},
-    //{"get_keepalive", meinheld_get_keepalive, METH_VARARGS, "return keep-alive support"},
+
+    {"set_keepalive", meinheld_set_keepalive, METH_VARARGS, "set keep-alive support. default 0. (disable keep-alive)"},
+    {"get_keepalive", meinheld_get_keepalive, METH_VARARGS, "return keep-alive support."},
+    
     {"set_max_content_length", meinheld_set_max_content_length, METH_VARARGS, "set max_content_length"},
     {"get_max_content_length", meinheld_get_max_content_length, METH_VARARGS, "return max_content_length"},
+    
     {"set_process_name", meinheld_set_process_name, METH_VARARGS, "set process name"},
     {"stop", meinheld_stop, METH_VARARGS, "stop main loop"},
     // support gunicorn 
@@ -1087,7 +1111,9 @@ initserver(void)
     PyType_Ready(&FileWrapperType);
     PyType_Ready(&ClientObjectType);
     PyType_Ready(&NSocketObjectType);
-
+#ifdef DEBUG
+    printf("client size %d \n", sizeof(client_t));
+#endif
 }
 
 
