@@ -25,7 +25,7 @@
 #include "http_parser.h"
 
 
-#define SERVER "meinheld/0.2.1"
+#define SERVER "meinheld/0.3"
 
 typedef enum {
     BODY_TYPE_NONE,
@@ -33,11 +33,15 @@ typedef enum {
     BODY_TYPE_BUFFER
 } request_body_type;
 
-
+/*
 typedef struct _client {
     int fd;
     char *remote_addr;
     uint32_t remote_port;
+    
+    uint8_t http_major;
+    uint8_t http_minor;
+
     uint8_t keep_alive;
     request *req;
     uint32_t body_length;
@@ -63,6 +67,7 @@ typedef struct _client {
     void *bucket;               //write_data
     uint8_t response_closed;    //response closed flag
 } client_t;
+*/
 
 extern int max_content_length; //max_content_length
 
